(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[9851],{66393:function(e,n,s){(window.__NEXT_P=window.__NEXT_P||[]).push(["/consent",function(){return s(78220)}])},77213:function(e,n,s){"use strict";s.d(n,{Z:function(){return f}});var t=s(24246),i=s(91922),r=s(88038),o=s.n(r),u=s(86677);s(27378);var c=s(25980),a=s(90867),l=s(77830),d=()=>{let e=(0,u.useRouter)();return(0,t.jsx)(i.xuv,{bg:"gray.50",border:"1px solid",borderColor:"blue.400",borderRadius:"md",justifyContent:"space-between",p:5,mb:5,mt:5,children:(0,t.jsxs)(i.xuv,{children:[(0,t.jsxs)(i.Kqy,{direction:{base:"column",sm:"row"},justifyContent:"space-between",children:[(0,t.jsx)(i.xvT,{fontWeight:"semibold",children:"Configure your storage and messaging provider"}),(0,t.jsx)(i.wpx,{onClick:()=>{e.push(l.fz)},children:"Configure"})]}),(0,t.jsxs)(i.xvT,{children:["Before Fides can process your privacy requests we need two simple steps to configure your storage and email client."," "]})]})})},f=e=>{let{children:n,title:s,padded:r=!0,mainProps:l}=e,f=(0,c.hz)(),h=(0,u.useRouter)(),p="/privacy-requests"===h.pathname||"/datastore-connection"===h.pathname,x=!(f.flags.privacyRequestsConfiguration&&p),{data:j}=(0,a.JE)(void 0,{skip:x}),{data:m}=(0,a.PW)(void 0,{skip:x}),v=f.flags.privacyRequestsConfiguration&&(!j||!m)&&p;return(0,t.jsxs)(i.kCb,{"data-testid":s,direction:"column",h:"100vh",children:[(0,t.jsxs)(o(),{children:[(0,t.jsxs)("title",{children:["Fides Admin UI - ",s]}),(0,t.jsx)("meta",{name:"description",content:"Privacy Engineering Platform"}),(0,t.jsx)("link",{rel:"icon",href:"/favicon.ico"})]}),(0,t.jsxs)(i.kCb,{as:"main",direction:"column",py:r?6:0,px:r?10:0,h:r?"calc(100% - 48px)":"full",flex:1,minWidth:0,overflow:"auto",...l,children:[v?(0,t.jsx)(d,{}):null,n]})]})}},78220:function(e,n,s){"use strict";s.r(n);var t=s(24246),i=s(91922),r=s(86677),o=s(27378),u=s(77213),c=s(77830);n.default=()=>{let e=(0,r.useRouter)();return(0,o.useEffect)(()=>{e.push(c.IT)},[e]),(0,t.jsx)(u.Z,{title:"Consent",children:(0,t.jsx)(i.M5Y,{children:(0,t.jsx)(i.$jN,{})})})}}},function(e){e.O(0,[2888,9774,179],function(){return e(e.s=66393)}),_N_E=e.O()}]);