"""Add fidesctl_meta to System

Revision ID: edcd28ede1f7
Revises: 50b3736634d2
Create Date: 2022-02-17 00:41:41.595166

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "edcd28ede1f7"
down_revision = "50b3736634d2"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("systems", sa.Column("fidesctl_meta", sa.JSON(), nullable=True))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("systems", "fidesctl_meta")
    # ### end Alembic commands ###
