"""add_system_vendor_id

Revision ID: 2af81f2b1a6f
Revises: b3b407ce2d5f
Create Date: 2023-11-06 21:46:30.715273

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "2af81f2b1a6f"
down_revision = "b3b407ce2d5f"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "ctl_systems", sa.Column("previous_vendor_id", sa.String(), nullable=True)
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("ctl_systems", "previous_vendor_id")
    # ### end Alembic commands ###
