(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[4290],{73906:function(e,n,o){(window.__NEXT_P=window.__NEXT_P||[]).push(["/consent/configure/add-vendors",function(){return o(32805)}])},32805:function(e,n,o){"use strict";o.r(n);var s=o(24246),t=o(91922),r=o(77213),a=o(77830),i=o(58754),c=o(15826);let{Text:d}=t.AntTypography;n.default=()=>(0,s.jsxs)(r.Z,{title:"Choose vendors",children:[(0,s.jsx)(i.Z,{heading:"Vendors",breadcrumbItems:[{title:"All vendors",href:a.zo},{title:"Choose vendors"}],children:(0,s.jsx)(d,{className:"block w-1/2",children:"Select your vendors below and they will be added as systems to your data map. Fides Compass will automatically populate the system information so that you can quickly configure privacy requests and consent."})}),(0,s.jsx)(c.$,{redirectRoute:a.zo})]})}},function(e){e.O(0,[8033,6060,4481,401,8934,5826,2888,9774,179],function(){return e(e.s=73906)}),_N_E=e.O()}]);