.MonitorStatusCell_errorResultTimestamp__DQQWt{color:var(--fidesui-neutral-400)}.MonitorStatusCell_errorTagText___MiU0{font-size:var(--fidesui-font-size-sm);cursor:pointer}.MonitorStatusCell_errorTagText___MiU0:hover{text-decoration:underline}